
--[[
=head1 NAME

applets.Tetris.TetrisApplet - Tetris Game

=head1 DESCRIPTION

Tetris is a simple game for the Jive platform

=head1 FUNCTIONS

Applet related methods are described in L<jive.Applet>. TetrisApplet overrides the
following methods:

=cut
--]]


-- stuff we use
local ipairs, tostring = ipairs, tostring
local table = table

local oo               = require("loop.simple")

local math                   = require("math")

local Applet           = require("jive.Applet")
local Window           = require("jive.ui.Window")
local Surface          = require("jive.ui.Surface")
local Popup            = require("jive.ui.Popup")
local Icon             = require("jive.ui.Icon")
local Framework        = require("jive.ui.Framework")
local Audio	       = require("jive.ui.Audio")

local log              = require("jive.utils.log").addCategory("Gravity", jive.utils.log.DEBUG)

local EVENT_SCROLL     = jive.ui.EVENT_SCROLL
local EVENT_KEY_PRESS  = jive.ui.EVENT_KEY_PRESS
local EVENT_KEY_DOWN   = jive.ui.EVENT_KEY_DOWN
local KEY_BACK         = jive.ui.KEY_BACK
local KEY_LEFT         = jive.ui.KEY_LEFT
local KEY_RIGHT        = jive.ui.KEY_RIGHT
local KEY_UP           = jive.ui.KEY_UP
local KEY_DOWN         = jive.ui.KEY_DOWN
local KEY_ADD	       = jive.ui.KEY_ADD
local KEY_PLAY	       = jive.ui.KEY_PLAY
local KEY_GO	       = jive.ui.KEY_GO

local frame_counter = 0

local collided = false

local game_mode = 1

local FRAMERATE        = 20

local playerRadius = 5

local currentLevel

local srf = nil
local window = nil
local w, h = Framework:getScreenSize()
local body_type_planetoid = 1
local body_x = 1
local body_y = 2
local body_r = 3
local body_type = 4

-- an array of bodies. A body is { x, y , r , body_type_x } 

local bodies = {}

-- describes the player. { x , y , vx, vy, frame_counter }
local pc = {0, 0, 0, 0, 0} 

-- defines the colour of body types.
local colours = {0xffffffff}

local score = 0

local thrust_x = 0

local thrust_y = 0

local cursor = {}

cursor.x = 0
cursor.y = 0

local level_data = {   
	-- level 1
	{ {20, 20, 0, 0 , 600}, --player data
	  {{100,100, 40, body_type_planetoid}} --bodies data
  	},

	--level 2
	{ {20, 20, 0, 0 , 600}, --player data
	  {{100,100, 20, body_type_planetoid},
  	   {150,150, 30, body_type_planetoid}} --bodies data
  	},
	--level 3
	{ {20, 20, 0, 0 , 800}, --player data
	  {{80,80, 30, body_type_planetoid},
  	   {160,160, 10, body_type_planetoid},
   	   {10,120, 40, body_type_planetoid}} --bodies data
  	}
}

module(...)
oo.class(_M, Applet)

function displayName(self)
	return "Gravity"
end

function initPlayfield(self, load_level)
	collided 	= false
	game_mode 	= 1
		
	if load_level < 0 or load_level > #level_data then
		load_level = 1
	end
	currentLevel 	= load_level	
	score		= 0
	loadPlayer(self, load_level);
	loadBodies(self, load_level);

end

function loadPlayer(self, load_level)
	
	for i = 1,#level_data[load_level][1] do 
		pc[i] = level_data[load_level][1][i]
	end
	frame_counter = pc[5]

end

function loadBodies(self, load_level)
	for i = 1,#level_data[load_level][2] do 
		bodies[i] = {}
		for j = 1,#level_data[load_level][2][i] do 
			bodies[i][j] = level_data[load_level][2][i][j]

		end

	end
	
end

function openWindow(self)
	self.window = self:_window("Gravity")
	self:tieAndShowWindow(self.window)
	return self.window
end

	
function drawBodies(self)

	for i = 1,#bodies do 
		if bodies[i][body_type] == body_type_planetoid then
			srf:circle(bodies[i][body_x], bodies[i][body_y], bodies[i][body_r], colours[body_type_planetoid])
		end
	end
end

function drawPlayer(self)
	
	local m_x = pc[1]
	local m_y = pc[2]
	srf:circle(m_x, m_y, playerRadius, 0xffffff)
	if game_mode == 2 then
		srf:line(m_x, m_y, cursor.x, cursor.y, 0xff0000ff)
	end
	
end

function drawTimer(self)
	if frame_counter == 0 then
		initPlayfield(self, currentLevel + 1)
	end
	local percentage = (frame_counter) / pc[5] 
	local timer_width = w * percentage 
	srf:filledRectangle(0, h - 15, timer_width , h - 5, 0xffffff)  

end

function drawField(self)
	
	--draw the border
	srf:filledRectangle(0, 0 , w, h, 0x000000ff)
		
	--draw the bodies
	drawBodies()
		
	--draw the player
	drawPlayer()
	
	drawTimer()
	--draw the score
	
	self.bg:reDraw()

end

function skin(self, s)
	s.snake.layout = Window.noLayout
end

function accPlayer(self)

	local m_x = pc[1]
	local m_y = pc[2]
	if game_mode == 2 then
		local d_x = m_x - cursor.x
		local d_y = m_y - cursor.y
		local total_d = ( math.abs(d_x) + math.abs(d_y) )
		local x_y_ratio = d_x / total_d
		local y_x_ratio = d_y / total_d
		local total_v = math.sqrt( (d_x * d_x) + (d_y * d_y))
		pc[3] = total_v * x_y_ratio
		pc[4] = total_v * y_x_ratio
		return

	end

	for i = 1,#bodies do 
	
		local b_x = bodies[i][body_x]
		local b_y = bodies[i][body_y]

		local b_r = bodies[i][body_r] 
		local b_d_squared = ((b_x - m_x) * (b_x - m_x)) + ((b_y - m_y) * (b_y - m_y))

		if b_d_squared < ( (b_r + playerRadius) * (b_r + playerRadius) ) then
			collided = true
		end
		local total_v = (8*(b_r * b_r * b_r)) / (b_d_squared)

		local x_y_ratio = (b_x - m_x) / (math.abs(b_x - m_x) + math.abs(b_y - m_y)) 

		local y_x_ratio =  (b_y - m_y) / (math.abs(b_x - m_x) + math.abs(b_y - m_y))	

		pc[3] = pc[3] + (total_v * x_y_ratio / FRAMERATE)

		pc[4] = pc[4] + (total_v * y_x_ratio / FRAMERATE)

	end

end

function movePlayer(self)

	pc[1] = pc[1] + (pc[3] / FRAMERATE)
	pc[2] = pc[2] + (pc[4] / FRAMERATE)
	if pc[1] < 0 or pc[1] > w then
		pc[3] = pc[3] * (-1)
	end

	if pc[2] < 0 or pc[2] > h - 20 then
		pc[4] = pc[4] * (-1)
	end
	frame_counter = frame_counter - 1

end

--handle keys.

function handleKey(self)

	if game_mode == 3 then
		if direction == KEY_LEFT then
			thrust_x = thrust_x - 3
		elseif direction == KEY_RIGHT then
			thrust_x = thrust_x + 3
		elseif direction == KEY_DOWN then
			thrust_y = thrust_y + 3
		elseif direction == KEY_UP then
			thrust_y = thrust_y - 3
		end
	elseif game_mode == 2 then
		if direction == KEY_LEFT then
			cursor.x = cursor.x - 3
		elseif direction == KEY_RIGHT then
			cursor.x = cursor.x + 3
		elseif direction == KEY_DOWN then
			cursor.y = cursor.y + 3
		elseif direction == KEY_UP then
			cursor.y = cursor.y - 3
		elseif direction == KEY_GO then
			accPlayer(self)
			game_mode = 3
		end	
	elseif game_mode == 1 then
		if direction == KEY_LEFT then
			pc[1] = pc[1] - 3
		elseif direction == KEY_RIGHT then
			pc[1] = pc[1] + 3
		elseif direction == KEY_DOWN then
			pc[2] = pc[2] + 3
		elseif direction == KEY_UP then
			pc[2] = pc[2] - 3
		elseif direction == KEY_GO then
			cursor.x = pc[1]
			cursor.y = pc[2]
			game_mode = 2
		end
	end
end

function autoMove(self) 
	
	if game_mode == 3 then
		accPlayer(self)
		movePlayer(self)
	end

	if collided then
		initPlayfield(self, currentLevel)
	end
	drawField(self)


end

function keyEvent(self, evt)

	  local key = evt:getKeycode()
	  
	  if key == KEY_LEFT then
			direction = KEY_LEFT
		elseif key == KEY_RIGHT then
			direction = KEY_RIGHT
		elseif key == KEY_DOWN then
			direction = KEY_DOWN
		elseif key == KEY_UP then
			direction = KEY_UP
		elseif key == KEY_ADD then
			direction = KEY_LEFT
		elseif key == KEY_GO then
			direction = KEY_GO
		elseif key == KEY_PLAY then
			direction = KEY_RIGHT
		end

	handleKey(self)
end

function _window(self, ...)	
  	window = Window(self:displayName())

	srf = Surface:newRGBA(w, h)
	srf:filledRectangle(0, 0, w, h, 0x000000ff)
	self.bg = Icon("background", srf)

	window:addListener(EVENT_KEY_PRESS, 
		function(evt)
			if evt:getKeycode() == KEY_BACK then
				window:hide()
			end
		end
	)

	window:addListener(EVENT_KEY_DOWN, 
		function(evt)
			keyEvent(self, evt)
		end
	)

	window:addWidget(self.bg)

	initPlayfield(self, 1)

	self.bg:addAnimation(
		function()
		  self:autoMove()
		  self:drawField()
		end,
		FRAMERATE	
	)

	return window
end


--[[

=head1 LICENSE

Copyright 2007 Logitech Harmony,

Based on my tetris engine, in turn based on "snake" by Lukas Frey

Feel free to reuse!

=cut
--]]
